"""test color_operations.colorspace.py."""
# The MIT License (MIT)
#
# Copyright (c) 2015 Mapbox
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import math
from itertools import product

import numpy as np
import pytest
from colormath.color_conversions import convert_color
from colormath.color_objects import LabColor, LCHabColor, LuvColor, XYZColor, sRGBColor

# enums required to define src and dst for convert and convert_arr
# public scalar func
# public 3d array funcs
from color_operations.colorspace import ColorSpace as cs
from color_operations.colorspace import convert, convert_arr, saturate_rgb

try:
    from collections.abc import Iterable
except ImportError:
    from collections import Iterable

to_colormath = {
    cs.rgb: sRGBColor,
    cs.xyz: XYZColor,
    cs.lab: LabColor,
    cs.lch: LCHabColor,
    cs.luv: LuvColor,
}


tests = (
    # (rgb, expected_lch)
    ((0, 0, 0), (0, 0, 0)),
    ((1.0, 0, 0), (53.2, 104.6, 0.7)),
    ((0.392156, 0.776470, 0.164705), (71.7, 83.5, 2.3)),
    ((0.0392, 0.1960, 0.3529), (20.3517, 27.8757, -1.4612)),
    ((0.0456, 0.1929, 0.3941), (20.8945, 34.9429, -1.3244)),
    ((1.0, 1.0, 1.0), (100, 0, 2.8)),
)

test_tol = 1


@pytest.mark.parametrize("pair", tests)
def test_fixtures(pair):
    # use colormath to confirm test values
    rgb, lch = pair
    cmlch = convert_color(sRGBColor(*rgb), LCHabColor).get_value_tuple()

    assert _near(lch[0:2], cmlch[0:2], 0.2)

    if lch[0] < 99.999999:
        # If L == 100, the hue is indeterminate
        # Otherwise, normalize to [0, 2*pi] and compare
        h = lch[2] % (math.pi * 2)
        cmh = math.radians(cmlch[2]) % (math.pi * 2)
        assert _near([h], [cmh], 0.2)


def _near(a, b, tol):
    if not isinstance(tol, Iterable):
        tol = [tol] * len(a)

    for x, y, t in zip(a, b, tol):
        if abs(x - y) > t:
            return False
    return True


def _make_array(x, y, z, dtype="float64"):
    """make a 3, 1, 1 array"""
    return np.array([[[x]], [[y]], [[z]]]).astype(dtype)


@pytest.mark.parametrize("pair", tests)
def test_rgb2lch(pair):
    rgb, lch = pair
    alch = convert(*rgb, src=cs.rgb, dst=cs.lch)
    assert round(alch[0], 8) >= 0
    assert _near(alch, lch, (1.0, 1.0, 0.25))


@pytest.mark.parametrize("pair", tests)
def test_roundtrip(pair):
    rgb, lch = pair
    argb = convert(*convert(*rgb, src=cs.rgb, dst=cs.lch), src=cs.lch, dst=cs.rgb)
    for v in argb:
        assert v > -0.0001
        assert v < 1.0001
    assert _near(argb, rgb, 0.1)


@pytest.mark.parametrize("pair", tests)
def test_lch2rgb(pair):
    rgb, lch = pair
    argb = convert(*lch, src=cs.lch, dst=cs.rgb)
    assert _near(argb, rgb, (1.0, 1.0, 0.1))


@pytest.mark.parametrize("pair", tests)
def test_arr_rgb(pair):
    rgb, lch = pair
    rgb = _make_array(*rgb)
    lch = _make_array(*lch)
    assert np.allclose(convert_arr(rgb, cs.rgb, cs.lch), lch, atol=0.2)


@pytest.mark.parametrize("pair", tests)
def test_arr_lch(pair):
    rgb, lch = pair
    rgb = _make_array(*rgb)
    lch = _make_array(*lch)
    assert np.allclose(convert_arr(lch, cs.lch, cs.rgb), rgb, atol=0.2)


@pytest.mark.parametrize("pair", tests)
def test_saturation_1(pair):
    rgb, lch = pair
    rgb = _make_array(*rgb)
    assert np.allclose(saturate_rgb(rgb, 1.0), rgb, atol=0.2)


def test_saturation_bw():
    rgb = _make_array(0.392156, 0.776470, 0.164705)
    sat = saturate_rgb(rgb, 0.0)
    assert _near((sat[0, 0, 0],), (sat[1, 0, 0],), tol=0.1)
    assert _near((sat[1, 0, 0],), (sat[2, 0, 0],), tol=0.1)


def test_saturation():
    rgb = _make_array(0.392156, 0.776470, 0.164705)
    saturated = _make_array(0.3425, 0.78372, 0.0)
    assert np.allclose(saturate_rgb(rgb, 1.1), saturated, atol=0.2)

    rgb = _make_array(0.0392, 0.1960, 0.3529)
    saturated = _make_array(0.0456, 0.1929, 0.3941)
    assert np.allclose(saturate_rgb(rgb, 1.25), saturated, atol=0.2)


def test_bad_array_bands():
    bad = np.random.random((2, 3, 3))
    with pytest.raises(ValueError) as exc:
        saturate_rgb(bad, 1.1)
    assert "3 bands" in str(exc.value)

    with pytest.raises(ValueError) as exc:
        convert_arr(bad, cs.rgb, cs.lch)
    assert "3 bands" in str(exc.value)


def test_bad_array_dims():
    bad = np.random.random((3, 3))
    with pytest.raises(ValueError) as exc:
        saturate_rgb(bad, 1.1)
    assert "wrong number of dimensions" in str(exc.value)

    with pytest.raises(ValueError) as exc:
        convert_arr(bad, cs.rgb, cs.lch)
    assert "wrong number of dimensions" in str(exc.value)


def test_bad_array_type():
    bad = np.random.random((3, 3, 3)).astype("uint8")
    with pytest.raises(ValueError) as exc:
        saturate_rgb(bad, 1.1)
    assert "dtype mismatch" in str(exc.value)

    with pytest.raises(ValueError) as exc:
        convert_arr(bad, cs.rgb, cs.lch)
    assert "dtype mismatch" in str(exc.value)


def test_array_bad_colorspace():
    arr = np.random.random((3, 3))
    with pytest.raises(ValueError):
        convert_arr(arr, src="FOO", dst="RGB")

    with pytest.raises(ValueError):
        convert_arr(arr, src=999, dst=999)


def test_bad_colorspace_string():
    """String colorspaces raise ValueError"""
    with pytest.raises(ValueError):
        convert(0.1, 0.1, 0.1, src="FOO", dst="RGB")


def test_bad_colorspace_invalid_int():
    """Invalid colorspace integers raise ValueError"""
    with pytest.raises(ValueError):
        convert(0.1, 0.1, 0.1, src=999, dst=999)


def test_bad_colorspace_invalid_enum():
    """Invalid colorspace enum names raise AttributeError"""
    with pytest.raises(AttributeError):
        convert(0.1, 0.1, 0.1, src=cs.foo, dst=cs.bar)


def assert_color_roundtrip(color, src, dst, tolerance):
    """Asserts roundtrip of color correction within a given tolerance

    Helper function for tests below.
    """
    other = convert(*color, src=src, dst=dst)
    rio_roundtrip = convert(*other, src=dst, dst=src)

    if _near(rio_roundtrip, color, tol=tolerance):
        return True
    else:
        # Did not roundtrip properly, can colormath do it?
        src_cm = to_colormath[src]
        dst_cm = to_colormath[dst]

        cm_roundtrip = convert_color(
            convert_color(src_cm(*color), dst_cm, illuminant="d65"),
            src_cm,
            illuminant="d65",
        ).get_value_tuple()

        assert _near(rio_roundtrip, cm_roundtrip, tol=tolerance)


rgb_vals = [0.0, 0.01, 0.3, 0.5, 0.7, 0.99, 1.0]
rgb_colors = xyz_colors = list(product(rgb_vals, repeat=3))

# In parameterizing destination colorspaces we use a list comprehension,
# omitting the source colorspace.
# Test roundtrip from RGB to everything else


@pytest.mark.parametrize("color", rgb_colors)
@pytest.mark.parametrize("dst", [v for v in cs if v not in (cs.rgb,)])
@pytest.mark.parametrize("tolerance", [0.1])
def test_rgb_convert_roundtrip(color, dst, tolerance):
    assert_color_roundtrip(color, cs.rgb, dst, tolerance)



"""test color_operations.operations.py."""
# The MIT License (MIT)
#
# Copyright (c) 2015 Mapbox
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import numpy as np
import pytest

from color_operations.operations import (
    gamma,
    parse_operations,
    saturation,
    sigmoidal,
    simple_atmo,
    simple_atmo_opstring,
)
from color_operations.utils import to_math_type


@pytest.fixture
def arr():
    return to_math_type(
        np.array(
            [
                # red
                [[1, 2], [3, 4]],
                # green
                [[5, 6], [7, 8]],
                # blue
                [[9, 10], [11, 12]],
            ]
        ).astype("uint8")
        * 10
    )


@pytest.fixture
def arr_rgba():
    return to_math_type(
        np.array(
            [
                [[1, 2], [3, 4]],  # red
                [[5, 6], [7, 8]],  # green
                [[9, 10], [11, 12]],  # blue
                [[0, 0], [25.5, 25.5]],  # alpha
            ]
        ).astype("uint8")
        * 10
    )


def test_sigmoidal(arr):
    x = sigmoidal(arr, 10, 0.15)
    assert x[0][0][0] - 0.08056034 < 1e-4

    # contrast < 0
    x = sigmoidal(arr, -10, 0.15)
    assert x[0][0][0] - 0.020186627 < 1e-4

    # bias zero, make it a tiny epsilon
    x = sigmoidal(arr, 10, 0)
    assert x[0][0][0] - 0.19362122 < 1e-4

    # contrast zzero, arrays are equal
    x = sigmoidal(arr, 0, 0.15)
    assert np.array_equal(x, arr)

    # output contains NaN
    with pytest.raises(ValueError):
        x = sigmoidal(arr, 100, -0.5)

    # output is not within the range of 0..1
    with pytest.raises(ValueError):
        arr[0][0][0] = 1.0
        arr[0][0][1] = 2.0
        x = sigmoidal(arr, 10, -0.5)


def test_gamma(arr):
    x = gamma(arr, 0.95)
    assert x[0][0][0] - 0.033069782 < 1e-4
    # output is not within the range of 0..1
    with pytest.raises(ValueError):
        x = gamma(arr, -2.0)
    # test output contains inf and is out of range 0..1
    with pytest.raises(ValueError):
        x = gamma(arr, -0.001)
    # test output contains NaN
    with pytest.raises(ValueError):
        x = gamma(arr, np.nan)
    with pytest.raises(ValueError):
        x = gamma(arr * -1, 2.2)


def test_sat(arr):
    x = saturation(arr, 50)
    assert x[0][0][0] - 0.15860622 < 1e-4


def test_sat_rgba_direct(arr_rgba):
    # Anything but 3-band RGB will fail
    with pytest.raises(ValueError):
        saturation(arr_rgba, 50)
    with pytest.raises(ValueError):
        saturation(arr_rgba[0:2], 50)


def test_atmo(arr):
    x = simple_atmo(arr, 0.03, 10, 0.15)
    assert x[0][0][0] - 0.080560341 < 1e-4

    # Gamma output is not within the range 0..1
    with pytest.raises(ValueError):
        x = simple_atmo(arr, 2.0, 10, 0.15)

    # Sigmoidal contrast output contains NaN
    with pytest.raises(ValueError):
        x = simple_atmo(arr, 0.03, 1000, -0.15)


def test_parse_gamma(arr):
    f = parse_operations("gamma rgb 0.95")[0]
    assert np.array_equal(f(arr), gamma(arr, 0.95))


def test_parse_sigmoidal(arr):
    f = parse_operations("sigmoidal rgb 5 0.53")[0]
    assert np.array_equal(f(arr), sigmoidal(arr, contrast=5, bias=0.53))


def test_parse_multi(arr):
    f1, f2 = parse_operations("gamma rgb 0.95 sigmoidal rgb 35 0.13")
    assert np.array_equal(
        f2(f1(arr)), sigmoidal(gamma(arr, g=0.95), contrast=35, bias=0.13)
    )


def test_parse_comma(arr):
    # Commas are optional whitespace, treated like empty string
    f1, f2 = parse_operations("gamma r,g,b 0.95, sigmoidal r,g,b 35 0.13")
    assert np.array_equal(
        f2(f1(arr)), sigmoidal(gamma(arr, g=0.95), contrast=35, bias=0.13)
    )


def test_parse_saturation_rgb(arr):
    f = parse_operations("saturation 1.25")[0]
    assert np.allclose(f(arr), saturation(arr, 1.25))


def test_parse_rgba(arr, arr_rgba):
    f = parse_operations("gamma rg 0.95")[0]
    rgb = f(arr)
    assert rgb.shape[0] == 3

    rgba = f(arr_rgba)
    assert rgba.shape[0] == 4
    # rgb bands are same
    assert np.allclose(rgba[0:3], rgb[0:3])
    # alpha unaltered
    assert np.array_equal(rgba[3], arr_rgba[3])


def test_saturation_rgba(arr, arr_rgba):
    f = parse_operations("saturation 1.25")[0]

    satrgb = f(arr)
    assert satrgb.shape[0] == 3

    satrgba = f(arr_rgba)
    # Still rgba
    assert satrgba.shape[0] == 4
    # alpha is unaltered
    assert np.array_equal(satrgba[3], arr_rgba[3])
    # first 3 bands are same b/t rgb and rgba
    assert np.allclose(satrgba[0:3], satrgb[0:3])


def test_parse_bad_op():
    with pytest.raises(ValueError):
        parse_operations("foob 123")


def test_parse_bands(arr):
    fa = parse_operations("gamma 1,2 0.95")[0]
    fb = parse_operations("gamma Rg 0.95")[0]
    assert np.array_equal(fa(arr), fb(arr))

    with pytest.raises(ValueError):
        parse_operations("gamma 7,8,9 1.05")


def test_parse_multi_saturation_first(arr):
    f1, f2 = parse_operations("saturation 1.25 gamma rgb 0.95")
    assert np.array_equal(f2(f1(arr)), gamma(saturation(arr, 1.25), g=0.95))


def test_parse_multi_name(arr):
    f1, f2 = parse_operations("saturation 1.25 gamma rgb 0.95")
    assert f1.__name__ == "saturation"
    assert f2.__name__ == "gamma"


def test_simple_atmos_opstring(arr):
    x = simple_atmo(arr, 0.03, 10, 0.15)
    ops = simple_atmo_opstring(0.03, 10, 0.15)
    for op in parse_operations(ops):
        arr = op(arr)
    assert np.allclose(x, arr)



"""test color_operations.utils.py."""
# The MIT License (MIT)
#
# Copyright (c) 2015 Mapbox
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import numpy as np
import pytest

from color_operations.utils import magick_to_rio, math_type, scale_dtype, to_math_type


@pytest.fixture
def arr():
    return (
        np.array(
            [[[1, 2], [3, 4]], [[5, 6], [7, 8]], [[9, 10], [11, 12]], [[0, 0], [0, 0]]]
        ).astype("uint8")
        * 10
    )


def test_to_math_type(arr):
    x = to_math_type(arr)
    assert x.dtype == math_type
    assert x.max() <= 1.0
    assert x.min() >= 0.0


def test_scale_dtype():
    arr = np.array([0.0, 1.0]).astype(math_type)
    x = scale_dtype(arr, "uint8")
    assert x.max() == 255
    assert x.min() == 0
    x = scale_dtype(arr, "uint16")
    assert x.max() == 65535
    assert x.min() == 0


def test_scale_round_trip(arr):
    x = to_math_type(arr)
    y = scale_dtype(x, arr.dtype)
    assert np.array_equal(arr, y)


def test_magick_to_rio():
    ops = magick_to_rio(
        "-channel B -sigmoidal-contrast 4 -gamma 0.95 "
        "-channel r -gamma 1.10 "
        "-channel rgb -sigmoidal-contrast 1x55% "
        "-channel G -gamma 0.9 "
        "-modulate 100,125 "
        "+channel -sigmoidal-contrast 3,40% "
        "-modulate 222,135 "
    )

    assert ops == " ".join(
        [
            "sigmoidal B 4 0.5",
            "gamma B 0.95",
            "gamma R 1.10",
            "sigmoidal RGB 1 0.55",
            "gamma G 0.9",
            "saturation 1.25",
            "sigmoidal RGB 3 0.4",
            "saturation 1.35",
        ]
    )



"""Color utilities."""

# The MIT License (MIT)
#
# Copyright (c) 2015 Mapbox
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import re

import numpy

# The type to be used for all intermediate math
# operations. Should be a float because values will
# be scaled to the range 0..1 for all work.

math_type = numpy.float64

epsilon = numpy.finfo(math_type).eps


def to_math_type(arr):
    """Convert an array from native integer dtype range to 0..1 scaling down linearly."""
    max_int = numpy.iinfo(arr.dtype).max
    return arr.astype(math_type) / max_int


def scale_dtype(arr, dtype):
    """Convert an array from 0..1 to dtype, scaling up linearly."""
    max_int = numpy.iinfo(dtype).max
    return (arr * max_int).astype(dtype)


def magick_to_operations(convert_opts):  # noqa: C901
    """Translate a limited subset of imagemagick convert commands to color-operations operations.

    Parameters
    ----------
    convert_opts: String, imagemagick convert options

    Returns
    -------
    operations string, ordered rio color operations

    """
    ops = []

    def set_band(x):
        global bands

        if x.upper() == "RGB":
            x = "RGB"

        bands = x.upper()

    set_band("RGB")

    def append_sig(arg):
        global bands

        args = list(filter(None, re.split("[,x]+", arg)))

        if len(args) == 1:
            args.append(0.5)

        elif len(args) == 2:
            args[1] = float(args[1].replace("%", "")) / 100.0

        ops.append("sigmoidal {} {} {}".format(bands, *args))

    def append_gamma(arg):
        global bands
        ops.append("gamma {} {}".format(bands, arg))

    def append_sat(arg):
        args = list(filter(None, re.split("[,x]+", arg)))
        # ignore args[0]
        # convert to proportion
        prop = float(args[1]) / 100
        ops.append("saturation {}".format(prop))

    nextf = None
    for part in convert_opts.strip().split(" "):
        if part == "-channel":
            nextf = set_band

        elif part == "+channel":
            set_band("RGB")
            nextf = None

        elif part == "-sigmoidal-contrast":
            nextf = append_sig

        elif part == "-gamma":
            nextf = append_gamma

        elif part == "-modulate":
            nextf = append_sat

        else:
            if nextf:
                nextf(part)
            nextf = None

    return " ".join(ops)


magick_to_rio = magick_to_operations



"""color-operations"""

from .operations import gamma, parse_operations, saturation, sigmoidal
from .utils import magick_to_operations, scale_dtype, to_math_type

__version__ = "0.2.0"


__all__ = [
    "sigmoidal",
    "gamma",
    "saturation",
    "parse_operations",
    "to_math_type",
    "scale_dtype",
    "magick_to_operations",
]



"""Setup script."""

# The MIT License (MIT)

# Copyright (c) 2015 Mapbox

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys

from setuptools import find_packages, setup
from setuptools.extension import Extension

# Use Cython if available.
try:
    from Cython.Build import cythonize
except ImportError:
    cythonize = None

include_dirs = []
try:
    import numpy

    include_dirs.append(numpy.get_include())
except ImportError:
    print("Numpy and its headers are required to run setup(). Exiting.")
    sys.exit(1)


with open("README.md") as f:
    readme = f.read()


if cythonize and "clean" not in sys.argv:
    ext_modules = cythonize(
        [
            Extension(
                "color_operations.colorspace",
                ["color_operations/colorspace.pyx"],
                extra_compile_args=["-O2"],
            )
        ]
    )
else:
    ext_modules = [
        Extension("color_operations.colorspace", ["color_operations/colorspace.c"])
    ]


setup(
    name="color-operations",
    description="Apply basic color-oriented image operations.",
    long_description=readme,
    long_description_content_type="text/markdown",
    python_requires=">=3.9",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Cython",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Multimedia :: Graphics :: Graphics Conversion",
        "Topic :: Scientific/Engineering :: GIS",
    ],
    keywords="",
    author="Charlie Loyd",
    author_email="charlie@mapbox.com",
    maintainer="Vincent Sarago",
    maintainer_email="vincent@developmentseed.com",
    url="https://github.com/vincentsarago/color-operations",
    license="MIT",
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=["numpy"],
    ext_modules=ext_modules,
    include_dirs=include_dirs,
    extras_require={
        "test": [
            "pytest",
            "colormath==2.0.2",
            "pytest-cov",
        ],
        "dev": [
            "pre-commit",
            "bump-my-version",
        ],
    },
)
